@@ -1,8 +1,6 @@
 from vaspvis.unfold import make_kpath,removeDuplicateKpoints, find_K_from_k, save2VaspKPOINTS
 from vaspvis.unfold import convert
-from vaspvis.dos import Dos
-from vaspvis.standard import _figure_setup_dos
-from vaspvis.passivator_utils import _append_H, _cart2sph, _get_bot_index, _get_neighbors, _get_top_index,_sort_by_z, _sph2cart
+from vaspvis.passivator_utils import _append_H, _cart2sph, _get_bot_index, _get_neighbors, _get_top_index,_sort_by_z, _sph2cart, _center_slab
 from pymatgen.analysis.graphs import StructureGraph
 from pymatgen.analysis.local_env import JmolNN, CrystalNN, EconNN
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
@@ -11,7 +9,6 @@ from pymatgen.core.periodic_table import Element
 from pymatgen.io.vasp.outputs import Eigenval, BSVasprun
 from pymatgen.io.vasp.inputs import Incar, Kpoints
 from pymatgen.electronic_structure.core import Spin, Orbital
-from vaspvis.band import Band
 from pymatgen.core.lattice import Lattice
 from pymatgen.core.structure import Structure
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
@@ -29,6 +26,50 @@ import time
 import copy
 import os
 
+
+def group_layers(structure, atol=None):
+    """
+    This function will find the atom indices belonging to each unique atomic layer.
+
+    Parameters:
+        structure (pymatgen.core.structure.Structure): Slab structure
+        atol (float or None): Tolarence used for grouping the layers. Useful for grouping
+            layers in a structure with relaxed atomic positions.
+
+    Returns:
+        A list containing the indices of each layers.
+        A list of heights of each layers in fractional coordinates.
+    """
+    sites = structure.sites
+    zvals = np.array([site.c for site in sites])
+    unique_values = np.sort(np.unique(np.round(zvals, 3)))
+    diff = np.mean(np.diff(unique_values)) * 0.2
+
+    grouped = False
+    groups = []
+    group_heights = []
+    zvals_copy = copy.deepcopy(zvals)
+    while not grouped:
+        if len(zvals_copy) > 0:
+            if atol is None:
+                group_index = np.where(
+                    np.isclose(zvals, np.min(zvals_copy), atol=diff)
+                )[0]
+            else:
+                group_index = np.where(
+                    np.isclose(zvals, np.min(zvals_copy), atol=atol)
+                )[0]
+
+            group_heights.append(np.min(zvals_copy))
+            zvals_copy = np.delete(zvals_copy, np.where(
+                np.isin(zvals_copy, zvals[group_index]))[0])
+            groups.append(group_index)
+        else:
+            grouped = True
+
+    return groups, np.array(group_heights)
+
+
 def convert_slab(bulk_path, slab_path, index, output='POSCAR_unfold', generate=True, print_M=True):
     """
     This function rotates a slab structure so its transformation matrix
@@ -78,16 +119,18 @@ def generate_kpoints(M, high_symmetry_points, n, output='KPOINTS'):
     save2VaspKPOINTS(reducedK, output)
 
 
-def get_bandgap(folder, printbg=True):
+def get_bandgap(folder, printbg=True, return_vbm_cbm=False):
     """
     Determines the band gap from a band structure calculation
 
     Parameters:
         folder (str): Folder that contains the VASP input and outputs files
         printbg (bool): Determines if the band gap value is printed out or not.
+        return_vbm_cbm (bool): Determines if the vbm and cbm are returned.
 
     Returns:
-        Bandgap in eV
+        if return_vbm_cbm is False: The band gap is returned in eV
+        if return_vbm_cbm is True: The band gap, vbm, and cbm are returned in eV in that order
     """
     def _get_bandgap(eigenvalues, printbg=printbg):
         if np.sum(np.diff(np.sign(eigenvalues[:,:,0])) != 0) == 0:
@@ -100,11 +143,19 @@ def get_bandgap(folder, printbg=True):
             bg = cbm - vbm
         else:
             bg = 0
+            vbm = np.nan
+            cbm = np.nan
 
         if printbg:
             print(f'Bandgap = {np.round(bg, 3)} eV')
+            if return_vbm_cbm:
+                print(f'VBM = {np.round(vbm, 3)} eV')
+                print(f'CBM = {np.round(cbm, 3)} eV')
 
-        return bg
+        if return_vbm_cbm:
+            return bg, vbm, cbm
+        else:
+            return bg
 
     pre_loaded_bands = os.path.isfile(os.path.join(folder, 'eigenvalues.npy'))
     eigenval = Eigenval(os.path.join(folder, 'EIGENVAL'))
@@ -190,7 +241,16 @@ def get_bandgap(folder, printbg=True):
     return band_gap
 
 
-def passivator(struc, passivated_struc=None, top=True, bot=True, symmetrize=True, tol=0.3):
+def passivator(
+    struc,
+    passivated_struc=None,
+    top=True,
+    bot=True,
+    symmetrize=True,
+    tol=0.0001,
+    write_file=False,
+    output='POSCAR_pas',
+):
     """
     This function passivates the slabs with pseudohydrogen. The positions of the pseudo-
     hydrogen are determined by locating the bonds between the second to last and last
@@ -199,8 +259,10 @@ def passivator(struc, passivated_struc=None, top=True, bot=True, symmetrize=True
     structure is input, the location of the passivating layer will be used on the new structure.
 
     Parameters:
-        struc (pymatgen.core.Structure): Unpassivated Structure
-        passivated_struc (pymatgen.core.Structure): Structure whos passivation layer has already been relaxed
+        struc (str or pymatgen.core.structure.Structure): Unpassivated Structure or file path to
+            unpassivated structure.
+        passivated_struc (str or pymatgen.core.Structure): Structure or file path to a structure whos
+            passivation layer has already been relaxed
         top (bool): Determines if the top of the slab is passivated
         bot (bool): Determines if the bottom of the slab is passivated
         symmetrize (bool): Determines if the slab is symmetrized or not
@@ -211,6 +273,13 @@ def passivator(struc, passivated_struc=None, top=True, bot=True, symmetrize=True
         struc_pas (pymatgen.core.Structure): Passivated pymatgen structure
     """
 
+    if type(struc) == str:
+        struc = Structure.from_file(struc)
+    if type(passivated_struc) == str:
+        passivated_struc = Structure.from_file(passivated_struc)
+
+    struc, shift = _center_slab(struc)
+
     sorted_slab, z_positions = _sort_by_z(struc)
 
     if passivated_struc is None:
@@ -349,6 +418,10 @@ def passivator(struc, passivated_struc=None, top=True, bot=True, symmetrize=True
     sorted_slab.remove_sites(sites_to_delete)
 
     sorted_slab = sorted_slab.get_sorted_structure()
+    sorted_slab.translate_sites(range(len(sorted_slab)), [0, 0, -shift])
+
+    if write_file:
+        Poscar(sorted_slab).write_file(output)
 
     return sorted_slab
 
@@ -456,7 +529,7 @@ def generate_slab(
         passivate_top=True,
         passivate_bot=True,
         symmetrize=False,
-        tol=0.3,
+        tol=0.0001,
 ):
     """
     This function generates a slab structure.
@@ -547,6 +620,8 @@ def compare_dos_to_bulk(
     fill_slab=True,
     figsize=(4,3),
 ):
+    from vaspvis.dos import Dos
+    from vaspvis.standard import _figure_setup_dos
     fig, ax = plt.subplots(figsize=figsize, dpi=300)
     _figure_setup_dos(ax=ax, fontsize=12, energyaxis='x')
 
@@ -676,20 +751,21 @@ def compare_dos_to_bulk(
 
 
 if __name__ == "__main__":
-    area_diff = compare_dos_to_bulk(
-        bulk_folder='../../vaspvis_data/dos_InAs/',
-        slab_folder='../../vaspvis_data/slabdos',
-        atoms=[17,50],
-    )
-    #  slab = generate_slab(
-        #  bulk='../../../../projects/unfold_test/POSCAR_InSb_conv',
-        #  miller_index=[1,0,0],
-        #  layers=8,
-        #  vacuum=30,
-        #  passivate=True,
+    #  area_diff = compare_dos_to_bulk(
+        #  bulk_folder='../../vaspvis_data/dos_InAs/',
+        #  slab_folder='../../vaspvis_data/slabdos',
+        #  atoms=[17,50],
     #  )
+    slab = generate_slab(
+        bulk='../../../../projects/unfold_test/POSCAR_InSb_conv',
+        miller_index=[1,1,0],
+        layers=8,
+        vacuum=30,
+        passivate=True,
+    )
     #  slab = passivator(
-        #  struc=Poscar.from_file('./POSCAR_100_test').structure
+        #  struc=Poscar.from_file('./POSCAR_0').structure,
+        #  write_file=True,
     #  )
     #  Poscar(slab).write_file('POSCAR_pas')
     #  M = convert_slab(
